
var gl, program;
var myZeta = 0.0, myPhi = Math.PI/2.0, radius = 1.4, fovy = 1.4;

var texturesId = [];

function getWebGLContext() {

  var canvas = document.getElementById("myCanvas");

  try {
    return canvas.getContext("webgl2");
  }
  catch(e) {
  }

  return null;

}

function initShaders() {
    
  var vertexShader = gl.createShader(gl.VERTEX_SHADER);
  gl.shaderSource(vertexShader, document.getElementById("myVertexShader").text);
  gl.compileShader(vertexShader);
  if (!gl.getShaderParameter(vertexShader, gl.COMPILE_STATUS)) {
    alert(gl.getShaderInfoLog(vertexShader));
    return null;
  }
 
  var fragmentShader = gl.createShader(gl.FRAGMENT_SHADER);
  gl.shaderSource(fragmentShader, document.getElementById("myFragmentShader").text);
  gl.compileShader(fragmentShader);
  if (!gl.getShaderParameter(fragmentShader, gl.COMPILE_STATUS)) {
    alert(gl.getShaderInfoLog(fragmentShader));
    return null;
  }
    
  program = gl.createProgram();
  gl.attachShader(program, vertexShader);
  gl.attachShader(program, fragmentShader);
    
  gl.linkProgram(program);
    
  gl.useProgram(program);
    
  program.vertexPositionAttribute = gl.getAttribLocation ( program, "VertexPosition");
  program.vertexNormalAttribute   = gl.getAttribLocation ( program, "VertexNormal");

  gl.enableVertexAttribArray(program.vertexPositionAttribute);
  gl.enableVertexAttribArray(program.vertexNormalAttribute);
    
  program.modelViewMatrixIndex  = gl.getUniformLocation( program, "modelViewMatrix");
  program.normalMatrixIndex     = gl.getUniformLocation( program, "normalMatrix");
  program.projectionMatrixIndex = gl.getUniformLocation( program, "projectionMatrix");
  program.skyboxIndex           = gl.getUniformLocation( program, 'skybox');
  program.invTIndex             = gl.getUniformLocation( program, 'invT');
  program.myCubeMapTextureIndex = gl.getUniformLocation( program, 'myCubeMapTexture');
  
  gl.uniform1i(program.myCubeMapTextureIndex, 0);
  
}

function initRendering() {
    
  gl.clearColor(0.95,0.95,0.95,1.0);
  gl.enable(gl.DEPTH_TEST);
    
}

function initBuffers(model) {
    
  model.idBufferVertices = gl.createBuffer ();
  gl.bindBuffer (gl.ARRAY_BUFFER, model.idBufferVertices);
  gl.bufferData (gl.ARRAY_BUFFER, new Float32Array(model.vertices), gl.STATIC_DRAW);
    
  model.idBufferIndices = gl.createBuffer ();
  gl.bindBuffer (gl.ELEMENT_ARRAY_BUFFER, model.idBufferIndices);
  gl.bufferData (gl.ELEMENT_ARRAY_BUFFER, new Uint16Array(model.indices), gl.STATIC_DRAW);
    
}

function initPrimitives() {
    
  initBuffers(examplePlane);
  initBuffers(exampleCube);
  initBuffers(exampleCover);
  initBuffers(exampleCone);
  initBuffers(exampleCylinder);
  initBuffers(exampleSphere);
    
  myTorus = makeTorus(0.4, 1.0, 80, 120);
  initBuffers(myTorus);
    
}

function setShaderProjectionMatrix(projectionMatrix) {
    
  gl.uniformMatrix4fv(program.projectionMatrixIndex, false, projectionMatrix);
    
}

function setShaderModelViewMatrix(modelViewMatrix) {
    
  gl.uniformMatrix4fv(program.modelViewMatrixIndex, false, modelViewMatrix);
    
}

function setShaderNormalMatrix(normalMatrix) {
    
  gl.uniformMatrix3fv(program.normalMatrixIndex, false, normalMatrix);
    
}

function getNormalMatrix(modelViewMatrix) {
    
  return mat3.normalFromMat4(mat3.create(), modelViewMatrix);
    
}

function getProjectionMatrix() {
    
  return mat4.perspective(mat4.create(), fovy, 2.0, 0.1, 100.0);
  
}

function getCameraMatrix() {
    
  // coordenadas esféricas a rectangulares: https://en.wikipedia.org/wiki/Spherical_coordinate_system
  var x = radius * Math.sin(myPhi) * Math.sin(myZeta);
  var y = radius * Math.cos(myPhi);
  var z = radius * Math.sin(myPhi) * Math.cos(myZeta);

  return mat4.lookAt(mat4.create(), [x, y, z], [0, 0, 0], [0, 1, 0]);
  
}

function drawSolid(model) {
    
  gl.bindBuffer (gl.ARRAY_BUFFER, model.idBufferVertices);
  gl.vertexAttribPointer (program.vertexPositionAttribute,  3, gl.FLOAT, false, 8*4,   0);
  gl.vertexAttribPointer (program.vertexNormalAttribute,    3, gl.FLOAT, false, 8*4, 3*4);
    
  gl.bindBuffer   (gl.ELEMENT_ARRAY_BUFFER, model.idBufferIndices);
  gl.drawElements (gl.TRIANGLES, model.indices.length, gl.UNSIGNED_SHORT, 0);
    
}

function drawSkyboxAndReflectObject() {

  // calculo la posición del skybox, que coincide con la de la camara
  var x = radius * Math.sin(myPhi) * Math.sin(myZeta);
  var y = radius * Math.cos(myPhi);
  var z = radius * Math.sin(myPhi) * Math.cos(myZeta);
  
  // el skybox es un cubo muy grande situado en la posicion de la camara
  var modelMatrix      = mat4.create();
  var modelViewMatrix  = mat4.create();
  var aux              = mat4.create();

  mat4.fromTranslation     (aux, [x, y, z]);
  mat4.fromScaling         (modelMatrix, [100.0,100.0,100.0]);
  mat4.multiply            (modelMatrix, aux, modelMatrix);
  mat4.multiply            (modelViewMatrix, getCameraMatrix(), modelMatrix);
  setShaderModelViewMatrix (modelViewMatrix);

  gl.uniform1f (program.skyboxIndex, 1.0);
  drawSolid (exampleCube);

  // ahora el objeto reflejante  
  var invT = mat3.create();
  mat3.fromMat4(invT,modelViewMatrix);
  mat3.invert  (invT,invT);
  gl.uniformMatrix3fv(program.invTIndex, false, invT);

  mat4.fromScaling (modelMatrix, [2.5,2.5,2.5]);
  mat4.multiply    (modelViewMatrix, getCameraMatrix(), modelMatrix);
  setShaderModelViewMatrix (modelViewMatrix);
  setShaderNormalMatrix    (getNormalMatrix(modelViewMatrix));

  gl.uniform1f (program.skyboxIndex, 0.0);
  drawSolid (myTorus);

}

function drawScene() {
    
  // se inicializan los buffers de color y de profundidad
  gl.clear(gl.COLOR_BUFFER_BIT | gl.DEPTH_BUFFER_BIT);
    
  // se obtiene la matriz de transformacion de la proyeccion y se envia al shader
  setShaderProjectionMatrix(getProjectionMatrix());

  // se establece la textura en la unidad 0
  gl.activeTexture(gl.TEXTURE0);
  gl.bindTexture(gl.TEXTURE_CUBE_MAP, texturesId[0]);  
  
  drawSkyboxAndReflectObject();

}

function initHandlers() {
    
  var mouseDown = false;
  var lastMouseX;
  var lastMouseY;

  var canvas = document.getElementById("myCanvas");

  canvas.addEventListener("mousedown",
    function(event) {
      mouseDown  = true;
      lastMouseX = event.clientX;
      lastMouseY = event.clientY;
    },
    false);

  canvas.addEventListener("mouseup",
    function() {
      mouseDown = false;
    },
    false);
  
  canvas.addEventListener("wheel",
    function (event) {
      
      var delta = 0.0;

      if (event.deltaMode == 0)
        delta = event.deltaY * 0.001;
      else if (event.deltaMode == 1)
        delta = event.deltaY * 0.03;
      else
        delta = event.deltaY;

      if (event.shiftKey == 1) { // fovy
          
        fovy *= Math.exp(-delta)
        fovy = Math.max (0.1, Math.min(3.0, fovy));
        
//         htmlFovy.innerHTML = (fovy * 180 / Math.PI).toFixed(1);
        
      } else {
        
        radius *= Math.exp(-delta);
        radius  = Math.max(Math.min(radius, 30), 0.05);
        
//         htmlRadius.innerHTML = radius.toFixed(1);
        
      }
      
      event.preventDefault();
      requestAnimationFrame(drawScene);

    }, false);

  canvas.addEventListener("mousemove",
    function (event) {
      
      if (!mouseDown) {
        return;
      }
      
      var newX = event.clientX;
      var newY = event.clientY;
      
      myZeta -= (newX - lastMouseX) * 0.005;
      myPhi  -= (newY - lastMouseY) * 0.005;
        
      var margen = 0.01;
      myPhi = Math.min (Math.max(myPhi, margen), Math.PI - margen);
        
//       htmlPhi.innerHTML  = (myPhi  * 180 / Math.PI).toFixed(1);
//       htmlZeta.innerHTML = (myZeta * 180 / Math.PI).toFixed(1);
     
      lastMouseX = newX
      lastMouseY = newY;
      
      event.preventDefault();
      requestAnimationFrame(drawScene);
      
    },
    false);

}

function isPowerOfTwo(x) {

  return (x & (x - 1)) == 0;

}

function nextHighestPowerOfTwo(x) {

  --x;
    
  for (var i = 1; i < 32; i <<= 1) {
    x = x | x >> i;
  }
    
  return x + 1;

}

function setCubeMapTexture(image, texturePos) {
  
  image.cont++;

  if (image.cont!= 6) return;
  
  var faces = [["posx.jpg", gl.TEXTURE_CUBE_MAP_POSITIVE_X],
               ["negx.jpg", gl.TEXTURE_CUBE_MAP_NEGATIVE_X],
               ["posy.jpg", gl.TEXTURE_CUBE_MAP_POSITIVE_Y],
               ["negy.jpg", gl.TEXTURE_CUBE_MAP_NEGATIVE_Y],
               ["posz.jpg", gl.TEXTURE_CUBE_MAP_POSITIVE_Z],
               ["negz.jpg", gl.TEXTURE_CUBE_MAP_NEGATIVE_Z]];

  // se indica el objeto textura
  gl.bindTexture(gl.TEXTURE_CUBE_MAP, texturesId[texturePos]);

  // parametros de filtrado
  gl.texParameteri(gl.TEXTURE_CUBE_MAP, gl.TEXTURE_MIN_FILTER, gl.LINEAR);
  gl.texParameteri(gl.TEXTURE_CUBE_MAP, gl.TEXTURE_MAG_FILTER, gl.LINEAR);
  
  // parametros de repeticion (coordenadas de textura mayores a uno)
  gl.texParameteri(gl.TEXTURE_CUBE_MAP, gl.TEXTURE_WRAP_S, gl.CLAMP_TO_EDGE);
  gl.texParameteri(gl.TEXTURE_CUBE_MAP, gl.TEXTURE_WRAP_T, gl.CLAMP_TO_EDGE);

  for (var i = 0; i < 6; i++) {

    // las hacemos potencia de dos si no lo son
    if (!isPowerOfTwo(image[i].width) || !isPowerOfTwo(image[i].height)) {
    
      // Scale up the texture to the next highest power of two dimensions.
      var canvas    = document.createElement("canvas");
      canvas.width  = nextHighestPowerOfTwo(image[i].width);
      canvas.height = nextHighestPowerOfTwo(image[i].height);
    
      var ctx       = canvas.getContext("2d");
      ctx.drawImage(image[i], 0, 0, canvas.width, canvas.height);
      image[i] = canvas;
    
    }
    //gl.pixelStorei(gl.UNPACK_FLIP_Y_WEBGL, 1);

    // datos de la textura
    gl.texImage2D (faces[i][1], 0, gl.RGB, gl.RGB, gl.UNSIGNED_BYTE, image[i]);
    
  }

  texturesId[texturePos].loaded = true;

  initHandlers();
  requestAnimationFrame(drawScene);
  
}

function loadCubeMapFromServer(folder, texturePos) {
  
  var image = [];
  var name  = [];

  image.cont = 0;

  for (var i = 0; i < 6; i++) {
  
    name[i]  = folder;
    image[i] = new Image();
    
    switch (i) {
      case 0 : name[i] += "posx.jpg"; break;
      case 1 : name[i] += "negx.jpg"; break;
      case 2 : name[i] += "posy.jpg"; break;
      case 3 : name[i] += "negy.jpg"; break;
      case 4 : name[i] += "posz.jpg"; break;
      case 5 : name[i] += "negz.jpg"; break;
    }

    image[i].addEventListener("load",
                       function() {
                       setCubeMapTexture(image, texturePos);
                       }, false);
    image[i].addEventListener("error",
                       function(err) {
                       console.log("MALA SUERTE: no esta disponible " + this.src);
                       }, false);
    image[i].crossOrigin = 'anonymous';
    image[i].src         = name[i];
  
  }
  
}

function initTextures() {

  var serverUrl  = "http://cphoto.uji.es/vj1221/assets/textures/";
  var texFolders = ["PereaBeach1/"];

  for (var texturePos = 0; texturePos < texFolders.length; texturePos++) {
  
    // creo el objeto textura
    texturesId[texturePos] = gl.createTexture();
    texturesId[texturePos].loaded = false;
    
    // solicito la carga de la textura
    loadCubeMapFromServer(serverUrl+texFolders[texturePos], texturePos);
    
  }

}

function initWebGL() {
    
  gl = getWebGLContext();
    
  if (!gl) {
    alert("WebGL 2.0 no está disponible");
    return;
  }
    
  initShaders();
  initPrimitives();
  initRendering();
  initTextures();
    
}

initWebGL();
